/**************************************************************************
  This is a library for several Adafruit displays based on ST77* drivers.

  Works with the Adafruit 1.8" TFT Breakout w/SD card
    ----> http://www.adafruit.com/products/358
  The 1.8" TFT shield
    ----> https://www.adafruit.com/product/802
  The 1.44" TFT breakout
    ----> https://www.adafruit.com/product/2088
  The 1.14" TFT breakout
  ----> https://www.adafruit.com/product/4383
  The 1.3" TFT breakout
  ----> https://www.adafruit.com/product/4313
  The 1.54" TFT breakout
    ----> https://www.adafruit.com/product/3787
  The 2.0" TFT breakout
    ----> https://www.adafruit.com/product/4311
  as well as Adafruit raw 1.8" TFT display
    ----> http://www.adafruit.com/products/618

  Check out the links above for our tutorials and wiring diagrams.
  These displays use SPI to communicate, 4 or 5 pins are required to
  interface (RST is optional).

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
 **************************************************************************/


// Updated with realtime clock DS3231

// Updated 3/4/2020 with daylight savings option

#include <Adafruit_GFX.h>    // Core graphics library
//#include <Adafruit_ST7735.h> // Hardware-specific library for ST7735
#include <Adafruit_ST7789.h> // Hardware-specific library for ST7789
#include <SPI.h>

#include "images.h"

#include "RTClib.h"

RTC_DS3231 rtc;

#include <Timezone.h> // for daylight savings rules

// US Eastern Time Zone (New York, Detroit)
//TimeChangeRule myDST = {"EDT", Second, Sun, Mar, 2, -240};    //Daylight time = UTC - 4 hours
//TimeChangeRule mySTD = {"EST", First, Sun, Nov, 2, -300};     //Standard time = UTC - 5 hours

// US Central Time Zone (Chicago, Austin)
TimeChangeRule myDST = {"EDT", Second, Sun, Mar, 2, -300};    //Daylight time = UTC - 5 hours
TimeChangeRule mySTD = {"EST", First, Sun, Nov, 2, -360};     //Standard time = UTC - 6 hours
Timezone myTZ(myDST, mySTD);

TimeChangeRule *tcr;        //pointer to the time change rule, use to get TZ abbrev


char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};


#if defined(ARDUINO_FEATHER_ESP32) // Feather Huzzah32
#define TFT_CS         14
#define TFT_RST        15
#define TFT_DC         32

#elif defined(ESP8266)
#define TFT_CS         4
#define TFT_RST        16
#define TFT_DC         5

#else
// For the breakout board, you can use any 2 or 3 pins.
// These pins will also work for the 1.8" TFT shield.
#define TFT_CS        -1
#define TFT_RST       11 // Or set to -1 and connect to Arduino RESET pin
#define TFT_DC         10
#endif

#define BACKLIGHT 13 // pin for the backlight

// OPTION 1 (recommended) is to use the HARDWARE SPI pins, which are unique
// to each board and not reassignable. For Arduino Uno: MOSI = pin 11 and
// SCLK = pin 13. This is the fastest mode of operation and is required if
// using the breakout board's microSD card.

// For 1.44" and 1.8" TFT with ST7735 use:
//Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);

// For 1.14", 1.3", 1.54", and 2.0" TFT with ST7789:
Adafruit_ST7789 tft = Adafruit_ST7789(TFT_CS, TFT_DC, TFT_RST);


// OPTION 2 lets you interface the display using ANY TWO or THREE PINS,
// tradeoff being that performance is not as fast as hardware SPI above.
//#define TFT_MOSI 7  // Data out
//#define TFT_SCLK 9  // Clock out

// For ST7735-based displays, we will use this call
//Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_MOSI, TFT_SCLK, TFT_RST);

// OR for the ST7789-based displays, we will use this call
//Adafruit_ST7789 tft = Adafruit_ST7789(TFT_CS, TFT_DC, TFT_MOSI, TFT_SCLK, TFT_RST);


float p = 3.1415926;




void setup(void) {

  // setup the serial connection for possible debugging.
  Serial.begin(9600);

  // setup the backlight pin
  pinMode(BACKLIGHT, OUTPUT);
  digitalWrite(BACKLIGHT, HIGH);

  delay(3000);

  // Set the backlight brightness using the PWM signal.  I use 8 for a dim light at night.  The maximum is 255.
  int brightness=8;
  analogWrite(BACKLIGHT, brightness);


  // Use this initializer if using a 1.8" TFT screen:
  //tft.initR(INITR_BLACKTAB);      // Init ST7735S chip, black tab

  // OR use this initializer if using a 1.8" TFT screen with offset such as WaveShare:
  // tft.initR(INITR_GREENTAB);      // Init ST7735S chip, green tab

  // OR use this initializer (uncomment) if using a 1.44" TFT:
  //tft.initR(INITR_144GREENTAB); // Init ST7735R chip, green tab

  // OR use this initializer (uncomment) if using a 0.96" 160x80 TFT:
  //tft.initR(INITR_MINI160x80);  // Init ST7735S mini display

  // OR use this initializer (uncomment) if using a 1.3" or 1.54" 240x240 TFT:
  //tft.init(240, 240);           // Init ST7789 240x240
  
  // Important Note: This generic display required using "SPI_MODE3"   ****
  tft.init(240, 240, SPI_MODE3);           // Init ST7789 240x240
  
  // OR use this initializer (uncomment) if using a 2.0" 320x240 TFT:
  //tft.init(240, 320);           // Init ST7789 320x240

  // OR use this initializer (uncomment) if using a 1.14" 240x135 TFT:
  //tft.init(135, 240);           // Init ST7789 240x135

  tft.setRotation(3); // sets the rotation direction of the display

  Serial.println("TFT is Initialized");

  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }


  tft.fillScreen(ST77XX_BLACK);

  // get the time from the real-time clock
  DateTime now = rtc.now();

  // show the date and time to the serial connection.
  Serial.print(now.year(), DEC);
  Serial.print('/');
  Serial.print(now.month(), DEC);
  Serial.print('/');
  Serial.print(now.day(), DEC);
  Serial.print(" (");
  Serial.print(daysOfTheWeek[now.dayOfTheWeek()]);
  Serial.print(") ");
  Serial.print(now.hour(), DEC);
  Serial.print(':');
  Serial.print(now.minute(), DEC);
  Serial.print(':');
  Serial.print(now.second(), DEC);
  Serial.println();

  //  uint16_t time = millis();
  //  time = millis() - time;

  //Serial.println(time, DEC);
  //delay(500);

  // clear the screen
  tft.fillScreen(ST77XX_BLACK);
  delay(100);


  // tft print function!
  tftPrintTest();
  delay(5000);

  tft.fillScreen(ST77XX_BLACK);


  delay(1000);
}

void loop() {

  DateTime timeNow = rtc.now();
  time_t local=myTZ.toLocal(timeNow.unixtime(), &tcr);
  
  plotTime( hour(local), minute(local) );
  delay(100);

}

void plotTime(int hour, int minutes) {

  //display the time.
  
  plotRectanglePartial( (hour % 12 ), minutes);
  drawSeconds(0,  minutes);
}



void plotRectangleImage (int hour) {

  // plots the whole image.
  // rewrite a version that controls how many rows to draw (start and stop rows);

  int xPixelOffset = 1; // offset from the corner of the display
  int yPixelOffset = 1;

  int xOffset = 2; // offset for the rectangular grid
  int yOffset = 2;

  int rectanglePixels = 13;
  int dividerPixels = 2;

  int xRectangles = 12;
  int yRectangles = 12;

  //tft.fillScreen(ST77XX_BLACK);

  for (int xCount = 0; xCount < xRectangles; xCount++) {
    for (int yCount = 0; yCount < yRectangles; yCount++) {

      char color1 = hourImages[hour][2 * ((xCount * xRectangles) + yCount) ];

      //      Serial.print("array index: ");
      //      Serial.print(2 * ((xCount * xRectangles) + yCount) );
      //
      //      Serial.print("  (x,y): (");
      //      Serial.print(xCount);
      //      Serial.print(",");
      //      Serial.print(yCount);
      //      Serial.print(")   ");
      //      Serial.print("Color1: ");
      //      Serial.print(color1, HEX);

      char color2 = hourImages[hour][1 + 2 * ((xCount * xRectangles) + yCount) ];

      //      Serial.print(" Color2 : ");
      //      Serial.print(color2, HEX);

      uint16_t thisColor = (color2 << 8) | (color1 & 0xff);

      //      Serial.print(" thisColor: ");
      //      Serial.println(thisColor, HEX);
      tft.fillRect(yPixelOffset + (dividerPixels + rectanglePixels) * (yCount + yOffset), xPixelOffset + (dividerPixels + rectanglePixels) * (xCount + xOffset), rectanglePixels, rectanglePixels, thisColor);
    }
  }


}

void plotRectanglePartial (int hour, int minutes) {

  int numberOfRows = 11 - (minutes / 5) ; // the number of rows to draw is dependent upon the five minute increments


  //  a version that controls how many rows to draw (from the bottom up);

  int xPixelOffset = 1; // offset from the corner of the display
  int yPixelOffset = 1;

  int xOffset = 2; // offset for the rectangular grid
  int yOffset = 2;

  int rectanglePixels = 13;
  int dividerPixels = 2;

  int xRectangles = 12;
  int yRectangles = 12;

  //tft.fillScreen(ST77XX_BLACK);

  for (int xCount = 0; xCount < xRectangles; xCount++) {
    for (int yCount = 0; yCount < yRectangles; yCount++) {
      int thisHour;

      if (xCount > numberOfRows) {
        thisHour = (hour) % 12;
      } else {
        if (hour == 0) {
          thisHour = 11;
        } else {
          thisHour = hour - 1;
        }
      }
      char color1 = hourImages[thisHour][2 * ((xCount * xRectangles) + yCount) ];

      //      Serial.print("array index: ");
      //      Serial.print(2 * ((xCount * xRectangles) + yCount) );
      //
      //      Serial.print("  (x,y): (");
      //      Serial.print(xCount);
      //      Serial.print(",");
      //      Serial.print(yCount);
      //      Serial.print(")   ");
      //      Serial.print("Color1: ");
      //      Serial.print(color1, HEX);

      char color2 = hourImages[thisHour][1 + 2 * ((xCount * xRectangles) + yCount) ];

      //      Serial.print(" Color2 : ");
      //      Serial.print(color2, HEX);

      uint16_t thisColor = (color2 << 8) | (color1 & 0xff);

      //      Serial.print(" thisColor: ");
      //      Serial.println(thisColor, HEX);
      tft.fillRect(yPixelOffset + (dividerPixels + rectanglePixels) * (yCount + yOffset), xPixelOffset + (dividerPixels + rectanglePixels) * (xCount + xOffset), rectanglePixels, rectanglePixels, thisColor);
    }
  }


}



void drawSeconds(int startSecond, int endSecond) {

  //int thisColor = ST77XX_WHITE; // Use white for the color
  int thisColor=ST77XX_RED;  // Use red for the outer color
  int blackColor = ST77XX_BLACK; // black for background

  for (int i = 0; i <= 8; i++) {
    if (i >= startSecond && i <= endSecond) {
      TFTdrawPixel(7 + i, 0, thisColor);
    } else {
      TFTdrawPixel(7 + i, 0, blackColor);
    }
  }

  for (int i = 9; i <= 23; i++) {
    if (i >= startSecond && i <= endSecond) {
      TFTdrawPixel(15, i - 8, thisColor);
    } else {
      TFTdrawPixel(15, i - 8, blackColor);
    }
  }

  for (int i = 24; i <= 38; i++) {
    if (i >= startSecond && i <= endSecond) {
      TFTdrawPixel(15 - (i - 23), 15, thisColor);
    } else {
      TFTdrawPixel(15 - (i - 23), 15, blackColor);
    }
  }

  for (int i = 39; i <= 53; i++) {
    if (i >= startSecond && i <= endSecond) {
      TFTdrawPixel(0, 15 - (i - 38), thisColor);
    } else {
      TFTdrawPixel(0, 15 - (i - 38), blackColor);
    }
  }


  for (int i = 54; i < 60; i++) {
    if (i >= startSecond && i <= endSecond) {
      TFTdrawPixel(i - 53, 0, thisColor);
    } else {
      TFTdrawPixel(i - 53, 0, blackColor);
    }
  }



  return;

}



void TFTdrawPixel(int x, int y, int thisColor) {

  // This function draws a single "megapixel" of the size shown below, 
  // including a black "divider" grid between each "megapixel"
  
  // This offsets the "megapixel" display starting point from the corner by this many pixels
  int xOffset = 2; // offset for the rectangular grid
  int yOffset = 2;

  int rectanglePixels = 13;
  int dividerPixels = 2;

  // draw a rectangle.

  tft.fillRect(xOffset + (dividerPixels + rectanglePixels)*x, yOffset + (dividerPixels + rectanglePixels)*y, rectanglePixels, rectanglePixels, thisColor);

}




// Helper files for the TimeZone adjustment and Daylight Savings time

// format and print a time_t value, with a time zone appended.
void printDateTime(time_t t, const char *tz)
{
  char buf[32];
  char m[4];    // temporary storage for month string (DateStrings.cpp uses shared buffer)
  strcpy(m, monthShortStr(month(t)));
  sprintf(buf, "%.2d:%.2d:%.2d %s %.2d %s %d %s",
          hour(t), minute(t), second(t), dayShortStr(weekday(t)), day(t), m, year(t), tz);
  Serial.println(buf);
}


/////////////////////////

/// These are some test routines from the original file from Adafruit

void testRectangles () {

  int xOffset = 1; // offset for the rectangular grid
  int yOffset = 1;

  int rectanglePixels = 13;
  int dividerPixels = 2;

  int xRectangles = 16;
  int yRectangles = 16;

  tft.fillScreen(ST77XX_BLACK);

  for (int xCount = 0; xCount < xRectangles; xCount++) {
    for (int yCount = 0; yCount < yRectangles; yCount++) {
      tft.fillRect(xOffset + (dividerPixels + rectanglePixels)*xCount, yOffset + (dividerPixels + rectanglePixels)*yCount, rectanglePixels, rectanglePixels, ST77XX_RED);
    }
  }


}

void testlines(uint16_t color) {
  tft.fillScreen(ST77XX_BLACK);
  for (int16_t x = 0; x < tft.width(); x += 6) {
    tft.drawLine(0, 0, x, tft.height() - 1, color);
    delay(0);
  }
  for (int16_t y = 0; y < tft.height(); y += 6) {
    tft.drawLine(0, 0, tft.width() - 1, y, color);
    delay(0);
  }

  tft.fillScreen(ST77XX_BLACK);
  for (int16_t x = 0; x < tft.width(); x += 6) {
    tft.drawLine(tft.width() - 1, 0, x, tft.height() - 1, color);
    delay(0);
  }
  for (int16_t y = 0; y < tft.height(); y += 6) {
    tft.drawLine(tft.width() - 1, 0, 0, y, color);
    delay(0);
  }

  tft.fillScreen(ST77XX_BLACK);
  for (int16_t x = 0; x < tft.width(); x += 6) {
    tft.drawLine(0, tft.height() - 1, x, 0, color);
    delay(0);
  }
  for (int16_t y = 0; y < tft.height(); y += 6) {
    tft.drawLine(0, tft.height() - 1, tft.width() - 1, y, color);
    delay(0);
  }

  tft.fillScreen(ST77XX_BLACK);
  for (int16_t x = 0; x < tft.width(); x += 6) {
    tft.drawLine(tft.width() - 1, tft.height() - 1, x, 0, color);
    delay(0);
  }
  for (int16_t y = 0; y < tft.height(); y += 6) {
    tft.drawLine(tft.width() - 1, tft.height() - 1, 0, y, color);
    delay(0);
  }
}

void testdrawtext(char *text, uint16_t color) {
  tft.setCursor(0, 0);
  tft.setTextColor(color);
  tft.setTextWrap(true);
  tft.print(text);
}

void testfastlines(uint16_t color1, uint16_t color2) {
  tft.fillScreen(ST77XX_BLACK);
  for (int16_t y = 0; y < tft.height(); y += 5) {
    tft.drawFastHLine(0, y, tft.width(), color1);
  }
  for (int16_t x = 0; x < tft.width(); x += 5) {
    tft.drawFastVLine(x, 0, tft.height(), color2);
  }
}

void testdrawrects(uint16_t color) {
  tft.fillScreen(ST77XX_BLACK);
  for (int16_t x = 0; x < tft.width(); x += 6) {
    tft.drawRect(tft.width() / 2 - x / 2, tft.height() / 2 - x / 2 , x, x, color);
  }
}

void testfillrects(uint16_t color1, uint16_t color2) {
  tft.fillScreen(ST77XX_BLACK);
  for (int16_t x = tft.width() - 1; x > 6; x -= 6) {
    tft.fillRect(tft.width() / 2 - x / 2, tft.height() / 2 - x / 2 , x, x, color1);
    tft.drawRect(tft.width() / 2 - x / 2, tft.height() / 2 - x / 2 , x, x, color2);
  }
}

void testfillcircles(uint8_t radius, uint16_t color) {
  for (int16_t x = radius; x < tft.width(); x += radius * 2) {
    for (int16_t y = radius; y < tft.height(); y += radius * 2) {
      tft.fillCircle(x, y, radius, color);
    }
  }
}

void testdrawcircles(uint8_t radius, uint16_t color) {
  for (int16_t x = 0; x < tft.width() + radius; x += radius * 2) {
    for (int16_t y = 0; y < tft.height() + radius; y += radius * 2) {
      tft.drawCircle(x, y, radius, color);
    }
  }
}

void testtriangles() {
  tft.fillScreen(ST77XX_BLACK);
  uint16_t color = 0xF800;
  int t;
  int w = tft.width() / 2;
  int x = tft.height() - 1;
  int y = 0;
  int z = tft.width();
  for (t = 0 ; t <= 15; t++) {
    tft.drawTriangle(w, y, y, x, z, x, color);
    x -= 4;
    y += 4;
    z -= 4;
    color += 100;
  }
}

void testroundrects() {
  tft.fillScreen(ST77XX_BLACK);
  uint16_t color = 100;
  int i;
  int t;
  for (t = 0 ; t <= 4; t += 1) {
    int x = 0;
    int y = 0;
    int w = tft.width() - 2;
    int h = tft.height() - 2;
    for (i = 0 ; i <= 16; i += 1) {
      tft.drawRoundRect(x, y, w, h, 5, color);
      x += 2;
      y += 3;
      w -= 4;
      h -= 6;
      color += 1100;
    }
    color += 100;
  }
}

void tftPrintTest() {

  tft.setTextWrap(false);
  tft.setTextColor(ST77XX_RED);
  tft.setTextSize(3);

  // print a special boot-up message for my wife.
  tft.println("");
  tft.println(" Happy");
  tft.println(" Valentine's");
  tft.println(" Day");
  tft.println(" Margaret!");
  tft.println("");
  tft.println("");
  tft.println("     2020");

  //  tft.setTextWrap(false);
  //  tft.fillScreen(ST77XX_BLACK);
  //  tft.setCursor(0, 30);
  //  tft.setTextColor(ST77XX_RED);
  //  tft.setTextSize(1);
  //  tft.println("Hello World!");
  //  tft.setTextColor(ST77XX_YELLOW);
  //  tft.setTextSize(2);
  //  tft.println("Hello World!");
  //  tft.setTextColor(ST77XX_GREEN);
  //  tft.setTextSize(3);
  //  tft.println("Hello World!");
  //  tft.setTextColor(ST77XX_BLUE);
  //  tft.setTextSize(4);
  //  tft.print(1234.567);
  //  delay(1500);
  //  tft.setCursor(0, 0);
  //  tft.fillScreen(ST77XX_BLACK);
  //  tft.setTextColor(ST77XX_WHITE);
  //  tft.setTextSize(0);
  //  tft.println("Hello World!");
  //  tft.setTextSize(1);
  //  tft.setTextColor(ST77XX_GREEN);
  //  tft.print(p, 6);
  //  tft.println(" Want pi ? ");
  //  tft.println(" ");
  //  tft.print(8675309, HEX); // print 8,675,309 out in HEX!
  //  tft.println(" Print HEX!");
  //  tft.println(" ");
  //  tft.setTextColor(ST77XX_WHITE);
  //  tft.println("Sketch has been");
  //  tft.println("running for : ");
  //  tft.setTextColor(ST77XX_MAGENTA);
  //  tft.print(millis() / 1000);
  //  tft.setTextColor(ST77XX_WHITE);
  //  tft.print(" seconds.");
}

void mediabuttons() {
  // play
  tft.fillScreen(ST77XX_BLACK);
  tft.fillRoundRect(25, 10, 78, 60, 8, ST77XX_WHITE);
  tft.fillTriangle(42, 20, 42, 60, 90, 40, ST77XX_RED);
  delay(500);
  // pause
  tft.fillRoundRect(25, 90, 78, 60, 8, ST77XX_WHITE);
  tft.fillRoundRect(39, 98, 20, 45, 5, ST77XX_GREEN);
  tft.fillRoundRect(69, 98, 20, 45, 5, ST77XX_GREEN);
  delay(500);
  // play color
  tft.fillTriangle(42, 20, 42, 60, 90, 40, ST77XX_BLUE);
  delay(50);
  // pause color
  tft.fillRoundRect(39, 98, 20, 45, 5, ST77XX_RED);
  tft.fillRoundRect(69, 98, 20, 45, 5, ST77XX_RED);
  // play color
  tft.fillTriangle(42, 20, 42, 60, 90, 40, ST77XX_GREEN);
}
